use cgmath::{Vector2, Vector3, Zero};
use utilities::{impl_reprc, prelude::*};
use vulkan_rs::prelude::*;

impl_reprc!(
    #[derive(Copy, Debug)]
    pub struct PositionNormalColorUV {
        #[assume_reprc]
        pub position: Vector3<f32>,

        #[assume_reprc]
        filler_p: f32,

        #[assume_reprc]
        pub normal: Vector3<f32>,

        #[assume_reprc]
        filler_n: f32,

        #[assume_reprc]
        pub uv: Vector2<f32>,

        #[assume_reprc]
        filler_u: [f32; 2],
    }
);

impl PositionNormalColorUV {
    pub fn new(position: Vector3<f32>, normal: Vector3<f32>, uv: Vector2<f32>) -> Self {
        Self {
            position,
            filler_p: 1.0,
            normal,
            filler_n: 0.0,
            uv,
            filler_u: [0.0, 0.0],
        }
    }
}

impl Default for PositionNormalColorUV {
    fn default() -> Self {
        PositionNormalColorUV {
            position: Vector3::zero(),
            filler_p: 1.0,
            normal: Vector3::zero(),
            filler_n: 0.0,
            uv: Vector2::zero(),
            filler_u: [0.0, 0.0],
        }
    }
}

impl VertexInputDescription for PositionNormalColorUV {
    fn attributes() -> Vec<VkVertexInputAttributeDescription> {
        vec![
            // position
            VkVertexInputAttributeDescription {
                location: 0,
                binding: 0,
                format: VK_FORMAT_R32G32B32_SFLOAT,
                offset: 0,
            },
            // normal
            VkVertexInputAttributeDescription {
                location: 1,
                binding: 0,
                format: VK_FORMAT_R32G32B32_SFLOAT,
                offset: 16,
            },
            // color_uv
            VkVertexInputAttributeDescription {
                location: 3,
                binding: 0,
                format: VK_FORMAT_R32G32_SFLOAT,
                offset: 32,
            },
        ]
    }
}
