pub mod directionallight;
pub mod light_wrapper;
pub mod pointlight;
pub mod spotlight;

use crate::prelude::*;
use anyhow::Result;
use std::sync::{Arc, Mutex};
use utilities::{impl_reprc, prelude::cgmath::Vector3};

impl_reprc!(
    #[derive(Copy)]
    struct ExtraInfo {
        #[assume_reprc]
        center: Vector3<f32>,
    }
);

pub trait RasterizerLightInterface<T: ReprC>: Sized {
    fn new(
        light: Light,
        device: &Arc<Device>,
        queue: &Arc<Mutex<Queue>>,
        image_count: usize,
        shadow_image_size: u32,
    ) -> Result<Self>;

    fn shadow_map(&self) -> &Arc<Image>;
    fn buffer(&self) -> &Arc<Buffer<T>>;
    fn eq(&self, light: &Light) -> bool;
    fn invalidate(&mut self);

    fn render(
        &mut self,
        content: &Vec<&EntityObject>,
        buffer_recorder: &mut CommandBufferRecorder<'_>,
        index: usize,
    ) -> Result<()>;
    fn set_center(&mut self, center: Vector3<f32>) -> Result<()>;
}
