#version 450

// input parameter
layout (location = 0) in vec3 in_world_pos;

#ifdef COLOR_UVS
    layout (location = 1) in vec2 in_color_uv;
#endif

#ifdef NORMALS
    #ifdef NORMAL_UVS
        layout (location = 2) in vec2 in_normal_uv;
        layout (location = 3) in vec3 in_tangent;
    #endif

    layout (location = 5) in vec3 in_normal;
#endif

#ifdef SHADOWED
    layout (location = 6) in vec4 in_shadow_coord;
#endif

layout (location = 0) out vec4 out_color;

// descriptors
layout (set = 0, binding = 1) uniform Material {
    vec4 color;
    float metallic_factor;
} material;

#ifdef COLOR_UVS
	layout (set = 0, binding = 2) uniform sampler2D color_texture;
#endif

#ifdef NORMAL_UVS
	layout (set = 0, binding = 4) uniform sampler2D normal_texture;
#endif

#ifdef SHADOWED
    layout (set = 2, binding = 0) uniform sampler2D directional_shadow_map;

    layout (set = 2, binding = 1) uniform DirectionalLight {
        vec4 direction;
        vec3 color;
        float distance;
        mat4 view;
    } directional_light_info;
#endif

bool texture_proj(vec4 P, sampler2D tex)
{
	vec4 shadow_coord = P / P.w;
	shadow_coord.st = shadow_coord.st * 0.5 + 0.5;

	if (shadow_coord.w > 0.0 && shadow_coord.z > -1.0 && shadow_coord.z < 1.0)
	{
		float dist = texture(tex, shadow_coord.st).r;

		if (dist < shadow_coord.z)
		{
			return true;
		}
	}

	return false;
}

void main() {
    #ifdef COLOR_UVS
        vec4 color = texture(color_texture, in_color_uv);
    #else
        vec4 color = material.color;
    #endif

    #ifdef SHADOWED
        bool shadowed = texture_proj(in_shadow_coord, directional_shadow_map);

        if (shadowed) {
            color.xyz *= 0.5;
        }
    #endif

    out_color = color;
}